package edu.kit.ipd.descartes.mm.applicationlevel.util;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import edu.kit.ipd.descartes.mm.applicationlevel.parameterdependencies.InfluencableModelVariable;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.AbstractAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BlackBoxBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.BranchAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.CoarseGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ComponentInternalBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCall;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ExternalCallFrequency;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.FineGrainedBehavior;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ForkAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.InternalAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.LoopAction;
import edu.kit.ipd.descartes.mm.applicationlevel.servicebehavior.ServiceBehaviorAbstraction;

public class NavigationUtil {
	public static List<Object> getObjectsWithClass(Collection<?> collection, Class<?> clazz) {
		List<Object> list = new LinkedList<Object>();
		for (Object o: collection)
			if (clazz.isInstance(o))
				list.add(o);
		return list;
	}

	public static List<?> getExternalCalls(ServiceBehaviorAbstraction behaviour) {
		if (behaviour instanceof CoarseGrainedBehavior) {
			List<Object> results = new LinkedList<Object>();
			List<ExternalCallFrequency> freqs = 
					((CoarseGrainedBehavior)behaviour).getExternalCallFrequency();
			for (ExternalCallFrequency freq: freqs) {
				if (freq.getExternalCall() != null)
					results.add(freq.getExternalCall());
			}
				return results;
		} else {
			FineGrainedBehavior beh = (FineGrainedBehavior)behaviour;
			return NavigationUtil.getExternalCalls(beh);
		}
	}

	private static List<ExternalCall> getExternalCalls(FineGrainedBehavior beh) {
		if (beh.getBehavior() != null)
			return getExternalCalls(beh.getBehavior());
		return Collections.emptyList();
	}
	
	private static List<ExternalCall> getExternalCalls(ComponentInternalBehavior beh) {
		List<ExternalCall> externalCalls = new LinkedList<ExternalCall>(); 
		for (AbstractAction aa: beh.getActions()) {
			if (aa instanceof ForkAction)  {
				ForkAction forkAction = (ForkAction)aa;
				for (ComponentInternalBehavior iBeh: forkAction.getForkedBehaviors())
					externalCalls.addAll(getExternalCalls(iBeh));
			}
			if (aa instanceof BranchAction) {
				BranchAction branchAction = (BranchAction)aa;
				for (ComponentInternalBehavior iBeh: branchAction.getBranches())
					externalCalls.addAll(getExternalCalls(iBeh));
			}
			if (aa instanceof LoopAction) {
				LoopAction loopAction = (LoopAction)aa;
				if (loopAction.getLoopBodyBehavior() != null) {
					ComponentInternalBehavior iBeh = loopAction.getLoopBodyBehavior();
					externalCalls.addAll(getExternalCalls(iBeh));
				}
			}
			if (aa instanceof ExternalCallAction) {
				ExternalCallAction externalCallAction = (ExternalCallAction)aa;
				if (externalCallAction.getExternalCall() != null)
					externalCalls.add(externalCallAction.getExternalCall());
			}
		}
		return externalCalls;
	}
	
	public static List<InfluencableModelVariable> getInfluencableModelVariables(
			ServiceBehaviorAbstraction beh) {
		if (beh instanceof BlackBoxBehavior)
			return getInfluencableModelVariables((BlackBoxBehavior)beh);
		if (beh instanceof CoarseGrainedBehavior)
			return getInfluencableModelVariables((CoarseGrainedBehavior)beh);
		if (beh instanceof FineGrainedBehavior)
			return getInfluencableModelVariables((FineGrainedBehavior)beh);
		return null;
	}
	
	private static List<InfluencableModelVariable> getInfluencableModelVariables(
			BlackBoxBehavior beh) {
		List<InfluencableModelVariable> ret = new LinkedList<InfluencableModelVariable>();
		if (beh.getResponseTime() != null)
			ret.add(beh.getResponseTime());
		return ret;
	}
	
	private static List<InfluencableModelVariable> getInfluencableModelVariables(
			CoarseGrainedBehavior beh) {
		List<InfluencableModelVariable> ret = new LinkedList<InfluencableModelVariable>();
		ret.addAll(beh.getResourceDemand());
		for (ExternalCallFrequency frequency : beh.getExternalCallFrequency())
			if (frequency.getFrequency() != null)
				ret.add(frequency.getFrequency());
		return ret;
	}
	
	private static List<InfluencableModelVariable> getInfluencableModelVariables(
			FineGrainedBehavior beh) {
		if (beh.getBehavior() != null)
			return getInfluencableModelVariables(beh.getBehavior());
		return Collections.emptyList();
	}

	private static List<InfluencableModelVariable> getInfluencableModelVariables(
			ComponentInternalBehavior behavior) {
		List<InfluencableModelVariable> variables = new LinkedList<InfluencableModelVariable>(); 
		for (AbstractAction aa: behavior.getActions()) {
			if (aa instanceof ForkAction)  {
				ForkAction forkAction = (ForkAction)aa;
				for (ComponentInternalBehavior iBeh: forkAction.getForkedBehaviors())
					variables.addAll(getInfluencableModelVariables(iBeh));
			}
			if (aa instanceof BranchAction) {
				BranchAction branchAction = (BranchAction)aa;
				if (branchAction.getBranchProbabilities() != null)
					variables.add(branchAction.getBranchProbabilities());
				for (ComponentInternalBehavior iBeh: branchAction.getBranches())
					variables.addAll(getInfluencableModelVariables(iBeh));
			}
			if (aa instanceof LoopAction) {
				LoopAction loopAction = (LoopAction)aa;
				if (loopAction.getLoopIterationCount() != null) 
					variables.add(loopAction.getLoopIterationCount());
				if (loopAction.getLoopBodyBehavior() != null) {
					ComponentInternalBehavior iBeh = loopAction.getLoopBodyBehavior();
					variables.addAll(getInfluencableModelVariables(iBeh));
				}
			}
			if (aa instanceof InternalAction) {
				InternalAction internalAction = (InternalAction)aa;
				if (internalAction.getResourceDemand() != null)
					variables.addAll(internalAction.getResourceDemand());
			}
		}
		return variables;
	}
}
